import imp
import os
import sys
from mock import Mock

import pytest

from libtaskotron import executor
from libtaskotron import exceptions as exc


class TestExecutorDirectiveLoader():

    def setup_method(self, method):
        self.ref_dir = os.path.abspath('testing/test_directives/')
        self.ref_taskdata = {}
        self.ref_inputdata = {'uuid': ''}
        self.ref_directivename = 'stub'

    def test_executor_load_directive(self, tmpdir):
        test_executor = executor.Executor(self.ref_taskdata, self.ref_inputdata, tmpdir)

        test_executor._load_directive(self.ref_directivename, self.ref_dir)

        assert '%s_directive' % self.ref_directivename in sys.modules.keys()

    def test_executor_load_directive_indict(self, tmpdir):
        test_executor = executor.Executor(self.ref_taskdata, self.ref_inputdata, tmpdir)

        test_executor._load_directive(self.ref_directivename, self.ref_dir)

        assert self.ref_directivename in test_executor.directives.keys()

    def test_executor_load_directive_notexist(self, tmpdir):
        self.ref_directivename = 'noexist'

        test_executor = executor.Executor(self.ref_taskdata, self.ref_inputdata, tmpdir)

        with pytest.raises(exc.TaskotronDirectiveError):
            test_executor._load_directive(self.ref_directivename, self.ref_dir)

    def test_executor_load_directive_our_importerror(self, tmpdir, monkeypatch):
        """ The idea here is to test the logic that looks for an ImportError
        during directive loading and only transforms the exception if it's coming
        from libtaskotron
        """

        stub_imp = Mock(side_effect=(ImportError("No module libtaskotron.stub")))
        monkeypatch.setattr(imp, 'load_source', stub_imp)

        test_executor = executor.Executor(self.ref_taskdata, self.ref_inputdata, tmpdir)

        with pytest.raises(exc.TaskotronImportError):
            test_executor._load_directive(self.ref_directivename, self.ref_dir)

    def test_executor_load_directive_notour_importerror(self, tmpdir, monkeypatch):
        """ The opposite case as out_importerror - make sure that the ImportError
        is not changed into a Taskotron exception if it's not from libtaskotron
        """
        stub_imp = Mock(side_effect=(ImportError("No module unicorn.candy")))
        monkeypatch.setattr(imp, 'load_source', stub_imp)

        test_executor = executor.Executor(self.ref_taskdata, self.ref_inputdata, tmpdir)

        with pytest.raises(ImportError):
            test_executor._load_directive(self.ref_directivename, self.ref_dir)
